from slapos.package import update, signature
import tempfile
import unittest

SIGNATURE = "-----BEGIN CERTIFICATE-----\nXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n-----END CERTIFICATE-----"

UPDATE_CFG_DATA = """
[slapupdate]
upgrade_key = slapos-upgrade-testing-key-with-config-file

[networkcache]
download-binary-cache-url = http://www.shacache.org/shacache
download-cache-url = https://www.shacache.org/shacache
download-binary-dir-url = http://www.shacache.org/shadir

signature-certificate-list = -----BEGIN CERTIFICATE-----
  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  -----END CERTIFICATE-----


"""

UPDATE_CFG_WITH_UPLOAD_DATA = UPDATE_CFG_DATA + """
signature_private_key_file = /etc/opt/slapos/signature.key
signature_certificate_file = /etc/opt/slapos/signature.cert
upload-cache-url = https://www.shacache.org/shacache
shacache-cert-file = /etc/opt/slapos/shacache.crt
shacache-key-file = /etc/opt/slapos/shacache.key
upload-dir-url = https://www.shacache.org/shadir
shadir-cert-file = /etc/opt/slapos/shacache.crt
shadir-key-file = /etc/opt/slapos/shacache.key
"""


class NetworkCacheTestCase(unittest.TestCase):

  def test_basic(self):
    info, self.configuration_file_path = tempfile.mkstemp()
    open(self.configuration_file_path, 'w').write(UPDATE_CFG_DATA)
    shacache = signature.NetworkCache(self.configuration_file_path)
    self.assertEqual(shacache.download_binary_cache_url,
                     "http://www.shacache.org/shacache")
    self.assertEqual(shacache.download_cache_url,
                     "https://www.shacache.org/shacache")
    self.assertEqual(shacache.download_binary_dir_url,
                     "http://www.shacache.org/shadir")

    self.assertEqual(shacache.signature_certificate_list,
                     SIGNATURE)            

    self.assertEqual(shacache.directory_key,
                     'slapos-upgrade-testing-key-with-config-file')
    # Check keys that don't exist
    # Not mandatory
    self.assertEqual(shacache.dir_url , None)
    self.assertEqual(shacache.cache_url , None)
    self.assertEqual(shacache.signature_private_key_file , None)
    self.assertEqual(shacache.shacache_cert_file , None)
    self.assertEqual(shacache.shacache_key_file , None)
    self.assertEqual(shacache.shadir_cert_file , None)
    self.assertEqual(shacache.shadir_key_file , None)


  def test_with_upload(self):
    info, self.configuration_file_path = tempfile.mkstemp()
    open(self.configuration_file_path, 'w').write(UPDATE_CFG_WITH_UPLOAD_DATA)
    shacache = signature.NetworkCache(self.configuration_file_path)
    self.assertEqual(shacache.download_binary_cache_url,
                     "http://www.shacache.org/shacache")
    self.assertEqual(shacache.download_cache_url,
                     "https://www.shacache.org/shacache")
    self.assertEqual(shacache.download_binary_dir_url,
                     "http://www.shacache.org/shadir")

    self.assertEqual(shacache.signature_certificate_list,
                     SIGNATURE)            

    self.assertEqual(shacache.directory_key,
                     'slapos-upgrade-testing-key-with-config-file')
    # Check keys that don't exist
    # Not mandatory
    self.assertEqual(shacache.dir_url , 'https://www.shacache.org/shadir')
    self.assertEqual(shacache.cache_url , 'https://www.shacache.org/shacache')
    self.assertEqual(shacache.shacache_cert_file , '/etc/opt/slapos/shacache.crt')
    self.assertEqual(shacache.shacache_key_file , '/etc/opt/slapos/shacache.key')
    self.assertEqual(shacache.shadir_cert_file , '/etc/opt/slapos/shacache.crt')
    self.assertEqual(shacache.shadir_key_file , '/etc/opt/slapos/shacache.key')

    self.assertEqual(shacache.signature_private_key_file ,'/etc/opt/slapos/signature.key')

  def test_file_dont_exist(self):
    self.assertRaises(ValueError, signature.NetworkCache, 
                          "/abc/123")








#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim: set et sts=2:
##############################################################################
#
# Copyright (c) 2012 Vifib SARL and Contributors.
# All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly advised to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import ConfigParser
import logging
from optparse import OptionParser, Option
import os
import sys
import tempfile
import urllib2


# create console handler and set level to debug
ch = logging.StreamHandler()
ch.setLevel(logging.WARNING)
# create formatter
formatter = logging.Formatter('%(name)s - %(levelname)s - %(message)s')
# add formatter to ch
ch.setFormatter(formatter)



class Parser(OptionParser):
  """
  Parse all arguments.
  """
  def __init__(self, usage=None, version=None):
    """
    Initialize all options possibles.
    """
    OptionParser.__init__(self, usage=usage, version=version,
                          option_list=[
      Option("--slapos-configuration",
             help="path to slapos configuration directory",
             default='/etc/opt/slapos/',
             type=str),
      Option("--slapos-cron",
             help="path to slapos cron file",
             default='/etc/cron.d/slapos-node',
             type=str),
      Option("--check-upload",
             help="Check if upload parameters are ok (do not check certificates)",
             default=False,
             action="store_true"),
      Option("--test-agent",
             help="Check if parameters are good for a test agent",
             default=False,
             action="store_true"),
      Option("-v","--verbose",
             default=False,
             action="store_true",
             help="Verbose output."),
      Option("-n", "--dry-run",
             help="Simulate the execution steps",
             default=False,
             action="store_true"),
   ])

  def check_args(self):
    """
    Check arguments
    """
    (options, args) = self.parse_args()
    if options.test_agent:
      options.check_upload = True
    return options


def get_slapos_conf_example():
  """
  Get slapos.cfg.example and return its path
  """
  register_server_url = "http://git.erp5.org/gitweb/slapos.core.git/blob_plain/HEAD:/slapos.cfg.example"
  request = urllib2.Request(register_server_url)
  url = urllib2.urlopen(request)
  page = url.read()
  info, path = tempfile.mkstemp()
  slapos_cfg_example = open(path,'w')
  slapos_cfg_example.write(page)
  slapos_cfg_example.close()
  return path


def check_networkcache(config, logger, configuration_parser,
                       configuration_example_parser):
  """
  Check network cache download
  """
  section = "networkcache"
  if configuration_parser.has_section(section):
    configuration_example_dict = dict(configuration_example_parser.items(section))
    configuration_dict = dict(configuration_parser.items(section))
    for key in configuration_example_dict:
      try:
        if not configuration_dict[key] ==  configuration_example_dict[key]:
          logger.warn("%s parameter in %s section is out of date" % (key, section))
      except KeyError:
        logger.warn("No %s parameter in %s section" % (key, section))
        pass

    if config.test_agent:
      configuration_dict = dict(configuration_parser.items('slapformat'))
      if int(configuration_dict['partition_amount']) < 60:
        logger.warn("Partition amount is to low for a test agent. Is %s but should be at least 60"
                    % configuration_dict['partition_amount'] )

    if config.check_upload == True:
      check_networkcache_upload(config, logger, configuration_dict)


class Upload:
  """
  Class used as a reference to check network cache upload
  """
  def __init__(self):
    self.data = {'download-binary-dir-url': 'http://www.shacache.org/shadir',
                 'signature_certificate_file': '/etc/slapos-cache/signature.cert',
                 'upload-dir-url': 'https://www.shacache.org/shadir',
                 'shadir-cert-file': '/etc/slapos-cache/shacache.cert',
                 'download-cache-url': 'https://www.shacache.org/shacache',
                 'upload-cache-url': 'https://www.shacache.org/shacache',
                 'shacache-cert-file': '/etc/slapos-cache/shacache.cert',
                 'upload-binary-cache-url': 'https://www.shacache.org/shacache',
                 'shacache-key-file': '/etc/slapos-cache/shacache.key',
                 'download-binary-cache-url': 'http://www.shacache.org/shacache',
                 'upload-binary-dir-url':'https://www.shacache.org/shadir',
                 'signature_private_key_file': '/etc/slapos-cache/signature.key',
                 'shadir-key-file': '/etc/slapos-cache/shacache.key'}


def check_networkcache_upload(config, logger, configuration_dict):
  """
  Check network cache upload
  """
  upload_parameters = Upload()
  for key in upload_parameters.data:
    try:
      if not key.find("file") == -1:
        file = configuration_dict[key]
        if not os.path.exists(file):
          logger.critical ("%s file for %s parameters does not exist "
                         % (file, key))
        else:
          logger.info ("%s parameter:%s does exists" % (key, file))
      else:
        if not configuration_dict[key] == upload_parameters.data[key]:
          logger.warn("%s is %s sould be %s"
                      % (
                          key,
                          configuration_dict[key],
                          upload_parameters.data[key]
                          )
                      )
    except KeyError:
      logger.critical("No %s parameter in networkcache section "
                        % (key))
      pass

def get_computer_name(certificate):
  """
  Extract computer_id for certificate
  """
  certificate = open(certificate,"r")
  for line in certificate:
    i = 0
    if "Subject" in line:
      k=line.find("COMP-")
      i=line.find("/email")
      certificate.close()
      return line[k:i]
  return -1

def check_computer_id(logger,computer_id,cert_file):
  """
  Get computer id from cert_file and compare with computer_id
  """
  comp_cert = get_computer_name(cert_file)
  if comp_cert == "":
    logger.error("Certificate file indicated is corrupted (no computer id)")
  elif comp_cert == computer_id:
      logger.info("Certificate and slapos.cfg define same computer id: %s"
                   % computer_id)
  else:
    logger.critical("Computers id from cerificate (%s) is different from slapos.cfg (%s)"
                    % (comp_cert,computer_id))

def slapos_conf_check (config):
  """
  Check if slapos.cfg look good
  """
  # Define logger for slapos.cfg verification
  logger = logging.getLogger('Checking slapos.cfg file:')
  logger.setLevel(logging.INFO)
  logger.addHandler(ch)
  # Load configuration file
  configuration_file_path = os.path.join (config.slapos_configuration
                                          ,'slapos.cfg')
  configuration_parser = ConfigParser.SafeConfigParser()
  configuration_parser.read(configuration_file_path)
  # Get example configuration file
  slapos_cfg_example = get_slapos_conf_example()
  configuration_example_parser = ConfigParser.RawConfigParser()
  configuration_example_parser.read(slapos_cfg_example)
  os.remove(slapos_cfg_example)

  # Check sections
  mandatory_sections = ["slapos","slapformat","networkcache"]
  for section in mandatory_sections:
    if not configuration_parser.has_section(section):
      logger.critical("No %s section in slapos.cfg" % section)
      mandatory_sections.remove(section)

  if 'networkcache' in mandatory_sections:
    mandatory_sections.remove('networkcache')

  # Check if parameters for slapos and slapformat exists
  for section in mandatory_sections:
    configuration_dict = dict(configuration_parser.items(section))
    configuration_example_dict = dict(configuration_example_parser.items(section))
    for key in configuration_example_dict:
      if not key in configuration_dict:
        logger.critical("No %s parameter in %s section "
                        % (key,section))
      # check if necessary files exist
      elif key in ("key_file","cert_file","certificate_repository_path"):
        files = configuration_dict[key]
        if not os.path.exists(files):
          logger.critical ("%s file for %s parameters does not exist "
                           % (files,key))
        else:
          logger.info ("%s parameter:%s does exists" % (key,files))
      # check if computer id is the same in slapos.cfg and certificate
      if key == "cert_file":
        check_computer_id(logger,configuration_dict["computer_id"],
                          configuration_dict["cert_file"])
  # Check networkcache
  check_networkcache(config,logger,configuration_parser,
                     configuration_example_parser)


class CronFile:
  """
  Class to analyse each cron line individualy
  """
  def __init__(self):
    """ Init all values"""
    self.slapformat  = -1
    self.slapgrid_cp = -1
    self.slapgrid_ur = -1
    self.slapgrid_sr = -1
    # cron file from slapos documentation
    self.slapgrid_sr_base = "* * * * * root /opt/slapos/bin/slapos node software --verbose --logfile=/opt/slapos/log/slapos-node-software.log > /dev/null 2>&1"
    self.slapgrid_cp_base = "* * * * * root /opt/slapos/bin/slapos node instance --verbose --logfile=/opt/slapos/log/slapos-node-instance.log > /dev/null 2>&1"
    self.slapgrid_ur_base = "0 * * * * root /opt/slapos/bin/slapos node report --maximal_delay=3600 --verbose --logfile=/opt/slapos/log/slapos-node-report.log > /dev/null 2>&1"
    self.slapformat_base = "0 * * * * root /opt/slapos/bin/slapos node format >> /opt/slapos/log/slapos-node-format.log 2>&1"


  def parse(self,cron_line):
    """ Parse cron line and give value to attributes """
    line = cron_line.split()
    if "slapos node format"  in cron_line:
      self.slapformat  = self.compare(self.slapformat,
                                      self.slapformat_base.split()  , line)
    if "slapos node report" in cron_line:
      self.slapgrid_ur = self.compare(self.slapgrid_ur,
                                      self.slapgrid_ur_base.split() , line)
    if "slapos node instance" in cron_line:
      self.slapgrid_cp = self.compare(self.slapgrid_cp,
                                      self.slapgrid_cp_base.split() , line)
    if "slapos node software" in cron_line:
      self.slapgrid_sr = self.compare(self.slapgrid_sr,
                                      self.slapgrid_sr_base.split() , line)

  def compare(self, state, reference, cron_line):
    if not state == -1:
      return 2
    for i in range(0, 6):
      if not reference[i] == cron_line[i]:
        return 0
    ref = len(set(reference[6:]))
    if not len(set(reference[6:]) & set(cron_line[6:])) == ref:
      return 0
    else: return 1

  def check(self, logger):
    elements = {
            'slapos node format': self.slapformat,
            'slapos node report': self.slapgrid_ur,
            'slapos node software': self.slapgrid_sr,
            'slapos node instance': self.slapgrid_cp
            }
    for key in elements:
      if elements[key] == 0:
        logger.error("Your line for '%s' command does not seem right" % key)
      elif elements[key] == -1:
        logger.error("No line found for '%s' command" % key)
      elif elements[key] == 1:
        logger.info("Line for '%s' command is good" % key)
      elif elements[key] == 2:
        logger.error("You have a duplicated line for '%s' command" % key)



def cron_check (config):
  """
  Check cron file
  """
  # Define logger for cron file verification
  logger = logging.getLogger('Checking slapos-node cron file:')
  logger.setLevel(logging.INFO)
  logger.addHandler(ch)
  cron = open(config.slapos_cron,"r")
  cron_file = CronFile()
  for line in cron:
    if "/opt/slapos" in line and not line[0]=="#":
      cron_file.parse(line)
  cron_file.check(logger)

def slapos_global_check (config):
  """
  Check for main files
  """
  # Define logger for computer chek
  logger = logging.getLogger('Checking your computer for SlapOS:')
  logger.setLevel(logging.INFO)
  logger.addHandler(ch)
  # checking slapos.cfg
  if not os.path.exists(os.path.join(config.slapos_configuration,'slapos.cfg')):
    logger.critical("No slapos.cfg found in slapos configuration directory: %s"
                    % config.slapos_configuration )
  else:
    logger.info("SlapOS configuration file found")
    slapos_conf_check(config)
  # checking cron file
  if not os.path.exists(config.slapos_cron):
    logger.warn("No %s found for cron" % config.slapos_cron)
  else:
    logger.info("Cron file found at %s" %config.slapos_cron)
    cron_check(config)

# Class containing all parameters needed for configuration
class Config:
  def setConfig(self, option_dict):
    """
    Set options given by parameters.
    """
    # Set options parameters
    for option, value in option_dict.__dict__.items():
      setattr(self, option, value)
    # Define logger for register
    self.logger = logging.getLogger('slapos-test configuration')
    self.logger.setLevel(logging.DEBUG)
    # add ch to logger
    self.logger.addHandler(ch)

    if self.verbose:
      ch.setLevel(logging.DEBUG)


  def displayUserConfig(self):
    self.logger.debug ("Slapos.cfg : %s" % self.slapos_configuration)
    self.logger.debug ("slapos cron file: %s" % self.slapos_cron)


def main():
  """Checking computer state to run slapos"""
  usage = "usage: %s [options] " % sys.argv[0]
  # Parse arguments
  config = Config()
  config.setConfig(Parser(usage=usage).check_args())
  config.displayUserConfig()
  slapos_global_check(config)
  sys.exit()


if __name__ == "__main__":
  main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2012 Vifib SARL and Contributors.
# All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly advised to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import ConfigParser
import datetime
import logging
from optparse import OptionParser, Option
import os
import subprocess as sub
import sys
import tempfile
from signature import Signature
from base_promise import BasePromise

# create console handler and set level to warning
ch = logging.StreamHandler()
ch.setLevel(logging.WARNING)
# create formatter
formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(name)s - %(message)s")
# add formatter to ch
ch.setFormatter(formatter)

class Parser(OptionParser):
  """
  Parse all arguments.
  """
  def __init__(self, usage=None, version=None):
    """
    Initialize all options possibles.
    """
    OptionParser.__init__(self, usage=usage, version=version,
                      option_list=[
        Option("--slapos-configuration",
               default='/etc/opt/slapos/slapos.cfg',
               help="Path to slapos configuration file"),
        Option("--srv-file",
               default='/srv/slapupdate',
               help="Server status file."),
        Option("-v", "--verbose",
               default=False,
               action="store_true",
               help="Verbose output."),
        Option("-n", "--dry-run",
               help="Simulate the execution steps",
               default=False,
               action="store_true"),
        ])

  def check_args(self):
    """
    Check arguments
    """
    (options, args) = self.parse_args()
    return options

# Class containing all parameters needed for configuration
class Config:
  def __init__(self, option_dict):
    # Set options parameters
    for option, value in option_dict.__dict__.items():
      setattr(self, option, value)

class Upgrader:
  def __init__(self, config_dict):
    # Set options parameters
    self.config = Config(config_dict)

    self.logger = logging.getLogger('Updating your machine')
    self.logger.setLevel(logging.DEBUG)
    # add ch to logger
    self.logger.addHandler(ch)

  def fixConsistency(self, signature, upgrade=0, reboot=0, boot=0, **kw):
    print upgrade, reboot, boot
    today = datetime.date.today().isoformat()
    if upgrade and boot:
      signature.update(reboot=today, upgrade=today)
    if upgrade:
      signature.update(upgrade=today)
    elif reboot:
      signature.update(reboot=today)
    else:
      raise ValueError(
        "You need upgrade and/or reboot when invoke fixConsistency!")

    if upgrade:
      pkgmanager = BasePromise()
      configuration_dict = signature.get_signature_dict()
      for entry in configuration_dict:
         signature_list = configuration_dict[entry].get("signature-list")
         if pkgmanager.matchSignatureList(signature_list):
           print "Upgrade FOUND!!!! %s " % entry
           upgrade_goal = configuration_dict[entry]
           break

      repository_tuple_list = []
      for repository in upgrade_goal['repository-list']:
        alias, url = repository.split("=")
        repository_tuple_list.append((alias.strip(), url.strip()))

      pkgmanager.update(repository_tuple_list, upgrade_goal['filter-package-list'])

  def checkConsistency(self, fixit=0, **kw):
  
    # Get configuration
    signature = Signature(self.config)
    signature.load()
  
    self.logger.debug("Expected Reboot early them %s" % signature.reboot)
    self.logger.debug("Expected Upgrade early them %s" % signature.upgrade)
    self.logger.debug("Last reboot : %s" % signature.last_reboot)
    self.logger.debug("Last upgrade : %s" % signature.last_upgrade)

    if signature.upgrade > datetime.date.today():
      self.logger.debug("Upgrade will happens on %s" % signature.upgrade)
      #return
 
    # Check if run for first time
    if signature.last_reboot is None:
      if fixit:
        # Purge repositories list and add new ones
        self.fixConsistency(signature, upgrade=1, boot=1)
    else:
      if signature.last_upgrade < signature.upgrade:
        # Purge repositories list and add new ones
        if fixit:
          self.fixConsistency(signature, upgrade=1)
      else:
        logger.info("Your system is up to date")
  
      if signature.last_reboot < signature.reboot:
        if not self.config.dry_run:
          self.fixConsistency(signature, reboot=1)
        else:
          self.logger.debug("Dry run: Rebooting required.")
  

  def run(self):
    """
    Will fetch information from web and update and/or reboot
    machine if needed
    """
    self.checkConsistency(fixit=not self.config.dry_run)
  
def main():
  """Update computer and slapos"""
  usage = "usage: %s [options] " % sys.argv[0]
  # Parse arguments
  upgrader = Upgrader(Parser(usage=usage).check_args())
  upgrader.run()
  sys.exit()

if __name__ == '__main__':
  main()




from slapos.package.base_promise import BasePromise
import os
import unittest

def _fake_call(self, *args, **kw):
  self.last_call = args

class testBasePromiseCase(unittest.TestCase):

  def setUp(self):
    BasePromise._call = _fake_call

  def testIsSystemd(self):
    promise = BasePromise()
    systemctl_path = "/tmp/_testBasePromiseCase_systemctl_test"
    promise.systemctl_path_list = ["/tmp/_testBasePromiseCase_systemctl_test"]
    if os.path.exists(systemctl_path):
      os.remove(systemctl_path)

    self.assertFalse(promise._isSystemd())

    open(systemctl_path, "w").write("echo test")
    self.assertTrue(promise._isSystemd())

  def testSystemctl(self):
    promise = BasePromise()
    def _fake_true_isSystemd():
      return True

    promise._isSystemd = _fake_true_isSystemd

    promise._service("service_name", "service_action")

    self.assertEqual(promise.last_call, 
             (['systemctl', 'service_action', 'service_name'],))

  def testService(self):
    promise = BasePromise()
    def _fake_false_isSystemd():
      return False

    promise._isSystemd = _fake_false_isSystemd

    promise._service("service_name", "service_action")

    self.assertEqual(promise.last_call, 
                     (['service', 'service_name', 'service_action'],))




#!/usr/bin/python
# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2012 Vifib SARL and Contributors.
# All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly advised to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import ConfigParser
import os
import time
import traceback
import tempfile
import datetime

from slapos.networkcachehelper import helper_download_network_cached_to_file
from slapos.networkcachehelper import helper_upload_network_cached_from_file


class NetworkCache:
  def __init__(self, configuration_path):
    if not os.path.exists(configuration_path):
      raise ValueError("You need configuration file")
    self.configuration = configuration_path
    self._load()
 
  def _load(self):
    network_cache_info = ConfigParser.RawConfigParser()
    network_cache_info.read(self.configuration)
    network_cache_info_dict = dict(network_cache_info.items('networkcache'))
    def get_(name):
      return network_cache_info_dict.get(name)

    self.download_binary_cache_url = get_('download-binary-cache-url')
    self.download_cache_url = get_('download-cache-url')
    self.download_binary_dir_url = get_('download-binary-dir-url')
    self.signature_certificate_list = get_('signature-certificate-list')

    # Not mandatory
    self.dir_url = get_('upload-dir-url')
    self.cache_url = get_('upload-cache-url')
    self.signature_private_key_file = get_('signature_private_key_file')
    self.shacache_cert_file = get_('shacache-cert-file')
    self.shacache_key_file = get_('shacache-key-file')
    self.shadir_cert_file = get_('shadir-cert-file')
    self.shadir_key_file = get_('shadir-key-file')


    if network_cache_info.has_section('slapupdate'):
      self.directory_key = network_cache_info.get('slapupdate', 'upgrade_key')
    else:
      self.directory_key = "slapos-upgrade-testing-key"

def get_yes_no(prompt):
  while True:
    answer = raw_input(prompt + " [y,n]: ")
    if answer.upper() in ['Y', 'YES']:
        return True
    if answer.upper() in ['N', 'NO']:
        return False

class Signature:

  def __init__(self, config, logger=None):
    self.config = config
    self.logger = logger

  def _download(self, path):
    """
    Download a tar of the repository from cache, and untar it.
    """
    shacache = NetworkCache(self.config.slapos_configuration)

    def strategy(entry_list):
      """Get the latest entry. """
      timestamp = 0
      best_entry = None
      for entry in entry_list:
        if entry['timestamp'] > timestamp:
          best_entry = entry

      return best_entry

    return helper_download_network_cached_to_file(
      path=path,
      directory_key=shacache.directory_key,
      required_key_list=['timestamp'],
      strategy=strategy,
      # Then we give a lot of not interesting things
      dir_url=shacache.download_binary_dir_url,
      cache_url=shacache.download_binary_cache_url,
      signature_certificate_list=shacache.signature_certificate_list,
    )
  
  def download(self):
    """
    Get status information and return its path
    """
    info, path = tempfile.mkstemp()
    if not self._download(path) == False:
      print open(path).read()
      return path
    else:
      raise ValueError("No result from shacache")

  def _upload(self, path):
    """
    Creates uploads repository to cache.
    """
    shacache = NetworkCache(self.config.slapos_configuration)
 
    metadata_dict = {
      # XXX: we set date from client side. It can be potentially dangerous
      # as it can be badly configured.
      'timestamp': time.time(),
    }
    try:
      if helper_upload_network_cached_from_file(
        path=path,
        directory_key=shacache.directory_key,
        metadata_dict=metadata_dict,
        # Then we give a lot of not interesting things
        dir_url=shacache.dir_url,
        cache_url=shacache.cache_url,
        signature_private_key_file=shacache.signature_private_key_file,
        shacache_cert_file=shacache.shacache_cert_file,
        shacache_key_file=shacache.shacache_key_file,
        shadir_cert_file=shacache.shadir_cert_file,
        shadir_key_file=shacache.shadir_key_file,
      ):
        print 'Uploaded update file to cache.'
    except Exception:
      print 'Unable to upload to cache:\n%s.' % traceback.format_exc()

  def upload(self, dry_run=0, verbose=1):
    upgrade_info = ConfigParser.RawConfigParser()
    upgrade_info.read(self.config.upgrade_file)

    tomorrow = datetime.date.today() + datetime.timedelta(days=1)

    if self.config.reboot:
      upgrade_info.set('system', 'reboot', tomorrow.isoformat())
    if self.config.upgrade:
      upgrade_info.set('system', 'upgrade', tomorrow.isoformat())

    file = open(self.config.upgrade_file, "w")
    upgrade_info.write(file)
    file.close()

    if verbose:
      print " You will update this :"
      print open(self.config.upgrade_file).read()

    if dry_run:
      return

    if get_yes_no("Do you want to continue? "):
      self._upload(self.config.upgrade_file)
 
  def update(self, reboot=None, upgrade=None):
    if reboot is None and upgrade is None:
      return 
    if not self.current_state.has_section('system'):
      self.current_state.add_section('system')

    if reboot is not None:
      self.current_state.set('system', 'reboot', reboot)

    if upgrade is not None:
      self.current_state.set('system', 'upgrade', upgrade)
  
    current_state_file = open(self.config.srv_file, "w")
    self.current_state.write(current_state_file)
    current_state_file.close()

  def get_signature_dict(self):
    """ Convert Next state info into a dict """
    map_dict = {}
    for key in self.next_state.sections():
      if key == "system":
        continue
      def clean_list(l):
        return [x.strip() for x in l.split('\n') if x.strip() != '']
      map_dict[key] = {}
      for entry in self.next_state.options(key):
        map_dict[key][entry] = clean_list(self.next_state.get(key, entry))

    return map_dict

  def _read_state(self, state, name):
    """ Extract information from config file """
    if not state.has_section('system'):
      return None
    return datetime.datetime.strptime(
      state.get('system', name), "%Y-%m-%d").date()

  def load(self):
    """
    Extract information from config file and server file
    """
    self.current_state = ConfigParser.RawConfigParser()
    self.current_state.read(self.config.srv_file)

    self.next_state = ConfigParser.ConfigParser()
    self.next_state.read(self.download())

    self.reboot = self._read_state(self.next_state, "reboot")
    self.upgrade = self._read_state(self.next_state, "upgrade")
    self.last_reboot = self._read_state(self.current_state, "reboot")
    self.last_upgrade = self._read_state(self.current_state, "upgrade")




import os
import subprocess
from distribution import PackageManager

class SlapError(Exception):
  """
  Slap error
  """
  def __init__(self, message):
    self.msg = message


class UsageError(SlapError):
  pass


class ExecError(SlapError):
  pass

class BasePromise(PackageManager):

  systemctl_path_list = ["/bin/systemctl", 
                         "/usr/bin/systemctl"]

  def log(self, message):
    """ For now only prints, but it is usefull for test purpose """
    print message

  def _isSystemd(self):
    """ Dectect if Systemd is used """
    for systemctl_path in self.systemctl_path_list:
      if os.path.exists(systemctl_path):
        return True
    return False

  def _service(self, name, action, stdout=None, stderr=None, dry_run=False):
    """
    Wrapper invokation of service or systemctl by identifying what it is available.
    """
    if self._isSystemd(): 
      self._call(['systemctl', action, name], stdout=stdout, stderr=stderr, dry_run=dry_run)
    else:
      self._call(['service', name, action], stdout=stdout, stderr=stderr, dry_run=dry_run)

  def _call(self, cmd_args, stdout=None, stderr=None, dry_run=False):
    """
    Wrapper for subprocess.call() which'll secure the usage of external program's.

    Args:
    cmd_args: list of strings representing the command and all it's needed args
    stdout/stderr: only precise PIPE (from subprocess) if you don't want the
    command to create output on the regular stream
    """
    self.log("Calling: %s" % ' '.join(cmd_args))

    if not dry_run:
      p = subprocess.Popen(cmd_args, stdout=stdout, stderr=stderr)
      output, err = p.communicate()
      return output, err

class BasePackagePromise(BasePromise):

  package_name = None
  binary_path = None

  def checkConsistency(self, fixit=0, **kw):
    is_ok = True
    if self.binary is not None and \
        not os.path.exists(self.binary_path):
      is_ok = False

    if self._isUpgradable(self.package_name):
      is_ok = False

    if not is_ok and fixit:
      return self.fixConsistency(**kw)

    return is_ok

  def fixConsistency(self, **kw):
    self._installSoftware(self.package_name)
    return self.checkConsistency(fixit=0, **kw)



import platform
import glob
import re
import os

_distributor_id_file_re = re.compile("(?:DISTRIB_ID\s*=)\s*(.*)", re.I)
_release_file_re = re.compile("(?:DISTRIB_RELEASE\s*=)\s*(.*)", re.I)
_codename_file_re = re.compile("(?:DISTRIB_CODENAME\s*=)\s*(.*)", re.I)

def patched_linux_distribution(distname='', version='', id='',
                               supported_dists=platform._supported_dists,
                               full_distribution_name=1):
    # check for the Debian/Ubuntu /etc/lsb-release file first, needed so
    # that the distribution doesn't get identified as Debian.
    try:
        etclsbrel = open("/etc/lsb-release", "rU")
        for line in etclsbrel:
            m = _distributor_id_file_re.search(line)
            if m:
                _u_distname = m.group(1).strip()
            m = _release_file_re.search(line)
            if m:
                _u_version = m.group(1).strip()
            m = _codename_file_re.search(line)
            if m:
                _u_id = m.group(1).strip()
        if _u_distname and _u_version:
            return (_u_distname, _u_version, _u_id)
    except (EnvironmentError, UnboundLocalError):
            pass

    return platform.linux_distribution(distname, version, id, supported_dists, full_distribution_name)

class PackageManager:

  def matchSignatureList(self, signature_list):
    return self.getOSSignature() in signature_list

  def getOSSignature(self):
    return "+++".join(patched_linux_distribution())

  def getDistributionName(self):
    return patched_linux_distribution()[0]

  def getVersion(self):
    return patched_linux_distribution()[1]

  def _call(self, *args, **kw):
    """ This is implemented in BasePromise """
    raise NotImplemented

  def _getDistribitionHandler(self):
    distribution_name = self.getDistributionName()
    if distribution_name.lower() == 'opensuse':
      return Zypper()

    elif distribution_name.lower() in ['debian', 'ubuntu']:
      return AptGet()

    raise NotImplemented("Distribution (%s) is not Supported!" % distribution_name) 

  def _purgeRepository(self):
    """ Remove all repositories """
    return self._getDistribitionHandler().purgeRepository(self._call)

  def _addRepository(self, url, alias):
    """ Add a repository """
    return self._getDistribitionHandler().addRepository(self._call, url, alias)

  def _updateRepository(self):
    """ Add a repository """
    return self._getDistribitionHandler().updateRepository(self._call)

  def _installSoftwareList(self, name_list):
    """ Upgrade softwares """
    return self._getDistribitionHandler().installSoftwareList(self._call, name_list)

  def _updateSoftware(self):
    """ Upgrade softwares """
    return self._getDistribitionHandler().updateSoftware(self._call)

  def _updateSystem(self):
    """ Dist-Upgrade of system """
    return self._getDistribitionHandler().updateSystem(self._call)

  def update(self, repository_list=[], package_list=[]):
    """ Perform upgrade """
    self._purgeRepository()
    for alias, url in repository_list:
      self._addRepository(url, alias)
    self._updateRepository()
    if len(package_list):
      self._installSoftwareList(package_list)

# This helper implements API for package handling
class AptGet:
 
  source_list_path = "/etc/apt/sources.list"
  source_list_d_path = "/etc/apt/sources.list.d"

  def purgeRepository(self, caller):
    """ Remove all repositories """
    # Aggressive removal
    os.remove(self.source_list_path)
    open("/etc/apt/sources.list", "w+").write("# Removed all")
    for file_path in glob.glob("%s/*" % self.source_list_d_path):
      os.remove(file_path)

  def addRepository(self, caller, url, alias):
    """ Add a repository """
    repos_file = open("%s/%s.list" % (self.source_list_d_path, alias), "w")
    prefix = "deb "
    if alias.endswith("-src"):
      prefix = "deb-src "
    repos_file.write(prefix + url)
    repos_file.close()

  def updateRepository(self, caller):
    """ Add a repository """
    caller(['apt-get', 'update'], stdout=None)

  def installSoftwareList(self, caller, name_list):
    """ Instal Software """
    self.updateRepository(caller)
    command_list = ["apt-get", "install", "-y"]
    command_list.extend(name_list)
    caller(command_list, stdout=None) 

  def isUpgradable(self, caller, name):
    output, err = caller(["apt-get", "upgrade", "--dry-run"])
    for line in output.splitlines():
      if line.startswith("Inst %s" % name):
        return True
    return False

  def updateSoftware(self, caller):
    """ Upgrade softwares """
    self.updateRepository(caller)
    caller(["apt-get", "upgrade"], stdout=None) 

  def updateSystem(self, caller):
    """ Dist-Upgrade of system """
    caller(['apt-get', 'dist-upgrade', '-y'], stdout=None)

class Zypper:
  def purgeRepository(self, caller):
    """Remove all repositories"""
    listing, err = caller(['zypper', 'lr'])
    while listing.count('\n') > 2:
      output, err = caller(['zypper', 'rr', '1'], stdout=None)
      listing, err = caller(['zypper', 'lr'])

  def addRepository(self, caller, url, alias):
    """ Add a repository """
    output, err = caller(['zypper', 'ar', '-fc', url, alias], stdout=None)

  def updateRepository(self, caller):
    """ Add a repository """
    caller(['zypper', '--gpg-auto-import-keys', 'up', '-Dly'], stdout=None)

  def isUpgradable(self, caller, name):
    output, err = caller(['zypper', '--gpg-auto-import-keys', 'up', '-ly'])
    for line in output.splitlines():
      if line.startswith("'%s' is already installed." % name):
        return False
    return True

  def installSoftwareList(self, caller, name_list):
    """ Instal Software """
    self.updateRepository(caller)
    command_list = ['zypper', '--gpg-auto-import-keys', 'up', '-ly']
    command_list.extend(name_list)
    caller(command_list, stdout=None) 

  def updateSoftware(self, caller):
    """ Upgrade softwares """
    caller(['zypper', '--gpg-auto-import-keys', 'up', '-ly'], stdout=None)

  def updateSystem(self, caller):
    """ Dist-Upgrade of system """
    caller(['zypper', '--gpg-auto-import-keys', 'dup', '-ly'], stdout=None)


def do_discover():
  package_manager = PackageManager()
  print package_manager.getOSSignature()





from setuptools import setup, find_packages

# Still under development
version = '0.0.1.3'
name = 'slapos.package'
long_description = open("README.txt").read() + "\n" + \
    open("CHANGES.txt").read() + "\n"

setup(name=name,
      version=version,
      description="SlapOS Package Utils",
      long_description=long_description,
      classifiers=[
          "Programming Language :: Python",
      ],
      keywords='slapos package update',
      license='GPLv3',
      url='http://www.slapos.org',
      author='VIFIB',
      namespace_packages=['slapos'],
      packages=find_packages(),
      include_package_data=True,
      install_requires=[
          'slapos.libnetworkcache',
          'iniparse',
      ],
      zip_safe=False,
      entry_points={
          'console_scripts': [
              # Those entry points are development version
              'slappkg-update = slapos.package.update:main',
              'slappkg-discover = slapos.package.distribution:do_discover',
              'slappkg-upload-key = slapos.package.upload_key:main'
          ],

        # Not supported yet
        #'slapos.cli': [
        #  'package upload-key = slapos.package.upload_key:main'
        #  ]
      },
      test_suite="slapos.package.test",
)
